@@ -1,7 +1,11 @@
+from __future__ import division
+
 import re
 import textwrap
 
 import wifi.subprocess_compat as subprocess
+from wifi.utils import db2dbm
+from wifi.exceptions import InterfaceError
 
 
 class Cell(object):
@@ -20,7 +24,13 @@ class Cell(object):
         """
         Returns a list of all cells extracted from the output of iwlist.
         """
-        iwlist_scan = subprocess.check_output(['/sbin/iwlist', interface, 'scan']).decode('utf-8')
+        try:
+            iwlist_scan = subprocess.check_output(['/sbin/iwlist', interface, 'scan'],
+                                                  stderr=subprocess.STDOUT)
+        except subprocess.CalledProcessError as e:
+            raise InterfaceError(e.output.strip())
+        else:
+            iwlist_scan = iwlist_scan.decode('utf-8')
         cells = map(Cell.from_string, cells_re.split(iwlist_scan)[1:])
 
         return cells
@@ -45,15 +55,9 @@ class Cell(object):
 cells_re = re.compile(r'Cell \d+ - ')
 quality_re_dict = {'dBm': re.compile(r'Quality=(\d+/\d+).*Signal level=(-\d+) dBm'),
                    'relative': re.compile(r'Quality=(\d+/\d+).*Signal level=(\d+/\d+)')}
-frequency_re = re.compile(r'([\d\.]+ .Hz).*')
+frequency_re = re.compile(r'([\d\.]+ .Hz).*Channel\s+(\d+).*')
 
 
-scalars = (
-    'address',
-    'channel',
-    'mode',
-)
-
 identity = lambda x: x
 
 key_translations = {
@@ -71,9 +75,7 @@ def normalize_key(key):
 
 normalize_value = {
     'ssid': lambda v: v.strip('"'),
-    'frequency': lambda v: frequency_re.search(v).group(1),
     'encrypted': lambda v: v == 'on',
-    'channel': int,
     'address': identity,
     'mode': identity,
 }
@@ -101,7 +103,7 @@ def normalize(cell_block):
                     cell.quality, signal = match_result.groups()
                     if re_name == 'relative':
                         actual, total = map(int, signal.split('/'))
-                        cell.signal = int((actual / total) * 100)
+                        cell.signal = db2dbm(int((actual / total) * 100))
                     else:
                         cell.signal = int(signal)
                     break
@@ -130,6 +132,12 @@ def normalize(cell_block):
 
                 if 'WPA2' in value:
                     cell.encryption_type = 'wpa2'
+                elif 'WPA' in value:
+                    cell.encryption_type = 'wpa'
+            if key == 'frequency':
+                frequency, channel = frequency_re.search(value).groups()
+                cell.frequency = frequency
+                cell.channel = int(channel)
             elif key in normalize_value:
                 setattr(cell, key, normalize_value[key](value))
 
